@@ -9,16 +9,17 @@ module Network.Wai.Handler.Warp.HTTP2 (
   , http2server
   ) where
 
-import qualified UnliftIO
 import qualified Data.ByteString as BS
-import Data.IORef (IORef, newIORef, writeIORef)
+import Data.IORef (IORef, newIORef, writeIORef, readIORef)
 import qualified Data.IORef as I
 import qualified Network.HTTP2.Frame as H2
 import qualified Network.HTTP2.Server as H2
 import Network.Socket (SockAddr)
+import Network.Socket.BufferPool
 import Network.Wai
 import Network.Wai.Internal (ResponseReceived(..))
 import qualified System.TimeManager as T
+import qualified UnliftIO
 
 import Network.Wai.Handler.Warp.HTTP2.File
 import Network.Wai.Handler.Warp.HTTP2.PushPromise
@@ -27,14 +28,14 @@ import Network.Wai.Handler.Warp.HTTP2.Response
 import Network.Wai.Handler.Warp.Imports
 import qualified Network.Wai.Handler.Warp.Settings as S
 import Network.Wai.Handler.Warp.Types
-import Network.Wai.Handler.Warp.Recv
 
 ----------------------------------------------------------------
 
 http2 :: S.Settings -> InternalInfo -> Connection -> Transport -> Application -> SockAddr -> T.Handle -> ByteString -> IO ()
 http2 settings ii conn transport app origAddr th bs = do
     istatus <- newIORef False
-    rawRecvN <- makeReceiveN bs (connRecv conn) (connRecvBuf conn)
+    rawRecvN <- makeRecvN bs $ connRecv conn
+    writeBuffer <- readIORef $ connWriteBuffer conn
     -- This thread becomes the sender in http2 library.
     -- In the case of event source, one request comes and one
     -- worker gets busy. But it is likely that the receiver does
@@ -45,8 +46,8 @@ http2 settings ii conn transport app origAddr th bs = do
     let recvN = wrappedRecvN th istatus (S.settingsSlowlorisSize settings) rawRecvN
         sendBS x = connSendAll conn x >> T.tickle th
         conf = H2.Config {
-            confWriteBuffer       = connWriteBuffer conn
-          , confBufferSize        = connBufferSize conn
+            confWriteBuffer       = bufBuffer writeBuffer
+          , confBufferSize        = bufSize writeBuffer
           , confSendAll           = sendBS
           , confReadN             = recvN
           , confPositionReadMaker = pReadMaker ii
@@ -114,7 +115,7 @@ http2server settings ii transport addr app h2req0 aux0 response = do
 
 wrappedRecvN :: T.Handle -> IORef Bool -> Int -> (BufSize -> IO ByteString) -> (BufSize -> IO ByteString)
 wrappedRecvN th istatus slowlorisSize readN bufsize = do
-    bs <- readN bufsize
+    bs <-  UnliftIO.handleAny handler $ readN bufsize
     unless (BS.null bs) $ do
         writeIORef istatus True
     -- TODO: think about the slowloris protection in HTTP2: current code
@@ -124,6 +125,9 @@ wrappedRecvN th istatus slowlorisSize readN bufsize = do
     -- deployments with large NATs may be trickier).
         when (BS.length bs >= slowlorisSize || bufsize <= slowlorisSize) $ T.tickle th
     return bs
+ where
+   handler :: UnliftIO.SomeException -> IO ByteString
+   handler _ = return ""
 
 -- connClose must not be called here since Run:fork calls it
 goaway :: Connection -> H2.ErrorCodeId -> ByteString -> IO ()
