@@ -4,9 +4,9 @@ module Network.Wai.Handler.Warp.Windows
   ) where
 
 #if WINDOWS
-import Control.Exception
 import Control.Concurrent.MVar
 import Control.Concurrent
+import qualified Control.Exception
 
 import Network.Wai.Handler.Warp.Imports
 
@@ -15,11 +15,12 @@ import Network.Wai.Handler.Warp.Imports
 -- @since 3.2.17
 windowsThreadBlockHack :: IO a -> IO a
 windowsThreadBlockHack act = do
-    var <- newEmptyMVar :: IO (MVar (Either SomeException a))
-    void . forkIO $ try act >>= putMVar var
+    var <- newEmptyMVar :: IO (MVar (Either Control.Exception.SomeException a))
+    -- Catch and rethrow even async exceptions, so don't bother with UnliftIO
+    void . forkIO $ Control.Exception.try act >>= putMVar var
     res <- takeMVar var
     case res of
-      Left  e -> throwIO e
+      Left  e -> Control.Exception.throwIO e
       Right r -> return r
 #else
 windowsThreadBlockHack :: IO a -> IO a
