@@ -2,12 +2,12 @@
 
 module WithApplicationSpec where
 
-import           Control.Exception
 import           Network.HTTP.Types
 import           Network.Wai
 import           System.Environment
 import           System.Process
 import           Test.Hspec
+import           UnliftIO.Exception
 
 import           Network.Wai.Handler.Warp.WithApplication
 
@@ -24,17 +24,17 @@ spec = do
         output `shouldBe` "foo"
 
     it "does not propagate exceptions from the server to the executing thread" $ do
-      let mkApp = return $ \ _request _respond -> throwIO $ ErrorCall "foo"
+      let mkApp = return $ \ _request _respond -> throwString "foo"
       withApplication mkApp $ \ port -> do
         output <- readProcess "curl" ["-s", "localhost:" ++ show port] ""
         output `shouldContain` "Something went wron"
 
   describe "testWithApplication" $ do
     it "propagates exceptions from the server to the executing thread" $ do
-      let mkApp = return $ \ _request _respond -> throwIO $ ErrorCall "foo"
+      let mkApp = return $ \ _request _respond -> throwString "foo"
       (testWithApplication mkApp $ \ port -> do
           readProcess "curl" ["-s", "localhost:" ++ show port] "")
-        `shouldThrow` (errorCall "foo")
+        `shouldThrow` (\(StringException str _) -> str == "foo")
 
 {- The future netwrok library will not export MkSocket.
   describe "withFreePort" $ do
